(function (Drupal, once, drupalSettings, debounce) {
  function setCookie(name, value, days) {
    const date = new Date();
    date.setTime(date.getTime() + days * 24 * 60 * 60 * 1000);
    const expires = `expires=${date.toUTCString()}`;
    document.cookie = `${name}=${value};${expires};path=/`;
  }

  function getCookie(name) {
    const match = document.cookie.match(new RegExp(`(^| )${name}=([^;]+)`));
    if (match) {
      return match[2];
    }
    return null;
  }

  function updateBannerOffset(banner) {
    const bannerOffsetHeight = banner.offsetHeight;
    document.documentElement.style.setProperty(
      '--eco_mode--banner--offset-top',
      `${bannerOffsetHeight}px`,
    );
  }

  function showBanner() {
    const banner = document.createElement('div');
    banner.className = 'response-banner';
    banner.id = 'responseBanner';
    banner.style.backgroundColor = drupalSettings.eco_mode.background_color;
    banner.style.color = drupalSettings.eco_mode.text_color;

    const closeButton = document.createElement('button');
    closeButton.classList.add('response-banner--close');
    closeButton.setAttribute('role', 'button');
    closeButton.setAttribute('aria-label', Drupal.t('Close eco mode banner'));
    closeButton.innerText = '✖';
    closeButton.addEventListener('click', function () {
      banner.style.display = 'none';
      setCookie('eco_mode_prompted', 1);
      updateBannerOffset(banner);
    });

    const bannerContent = document.createElement('div');
    bannerContent.className = 'banner-content';
    bannerContent.style.display = 'flex';
    bannerContent.style.justifyContent = 'space-between';
    bannerContent.style.alignItems = 'center';

    const htmlContainer = document.createElement('div');
    htmlContainer.className = 'banner-html-container';
    htmlContainer.style.marginBottom = '10px';
    htmlContainer.innerHTML = drupalSettings.eco_mode.message;

    const link = document.createElement('a');
    link.className = 'banner-link';
    link.href = `${drupalSettings.eco_mode.switch_url}?destination=${window.location.pathname}`;
    link.innerText = Drupal.t('Click here to switch to eco mode');
    link.style.color = drupalSettings.eco_mode.text_color;
    link.style.display = 'block';
    link.style.marginTop = '10px';

    banner.appendChild(closeButton);
    banner.appendChild(htmlContainer);
    banner.appendChild(link);
    document.body.appendChild(banner);
    return banner;
  }

  async function checkServerResponseTime(url, threshold) {
    try {
      const startTime = performance.now();
      await fetch(url, { method: 'HEAD' });
      const endTime = performance.now();
      const responseTime = endTime - startTime;
      setCookie('eco_mode_last_check', new Date().getTime() / 1000);
      if (responseTime > threshold) {
        return showBanner();
      }
    } catch (error) {
      console.error(error.message);
    }
  }

  Drupal.behaviors.testServerSpeedBehavior = {
    attach(context) {
      once('ecoMode', 'body', context).forEach(function () {
        if (
          getCookie('eco_mode_prompted') !== '1' &&
          getCookie('eco_mode') !== '1'
        ) {
          const urlToCheck = window.location.href;
          const responseTimeThreshold = drupalSettings.eco_mode.latency;
          const { interval } = drupalSettings.eco_mode;
          if (
            interval > 0 &&
            new Date().getTime() / 1000 - getCookie('eco_mode_last_check') <
              interval
          ) {
            return [];
          }

          const isAdminRoute = drupalSettings.eco_mode.is_admin_route;
          const isEcoModeEnabledForAdmin =
            drupalSettings.eco_mode.enabled_for_admin;
          // If the current route is an admin route and the eco-mode admin
          // route option is disabled, then do not check the
          // server response time.
          if (!isEcoModeEnabledForAdmin && isAdminRoute) {
            return [];
          }

          checkServerResponseTime(urlToCheck, responseTimeThreshold).then(
            (banner) => {
              if (banner) {
                updateBannerOffset(banner);
                window.addEventListener(
                  'resize',
                  debounce(() => {
                    updateBannerOffset(banner);
                  }, 200),
                );
              }
            },
          );
        }
      });
    },
  };
})(Drupal, once, drupalSettings, Drupal.debounce);
