<?php

namespace Drupal\eco_mode\Manager;

use Drupal\Core\Access\CsrfTokenGenerator;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\eco_mode\Trait\RenderManagerTrait;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Class EcoModeManager.
 *
 *  Eco mode manager.
 *
 * @package Drupal\eco_mode\Manager
 */
class EcoModeManager implements TrustedCallbackInterface, EcoModeManagerInterface {

  use RenderManagerTrait;

  const SERVICE_NAME = 'eco_mode.eco_mode_manager';

  const ECO_MODE_COOKIE = 'eco_mode';

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   *   The request stack.
   */
  protected RequestStack $request;

  /**
   * The cookie value.
   *
   * @var string|int|bool|float|null
   */
  protected string | int | bool | null | float $cookie;

  /**
   * The CSRF token generator.
   *
   * @var \Drupal\Core\Access\CsrfTokenGenerator
   */
  protected CsrfTokenGenerator $csrfTokenGenerator;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * EcoModeManager constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   The request stack.
   * @param \Drupal\Core\Access\CsrfTokenGenerator $csrfTokenGenerator
   *   The CSRF token generator.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(RequestStack $request, CsrfTokenGenerator $csrfTokenGenerator, ModuleHandlerInterface $moduleHandler, EntityTypeManagerInterface $entityTypeManager) {
    $this->request = $request;
    $this->cookie = $request->getCurrentRequest()->cookies->get(self::ECO_MODE_COOKIE);
    $this->csrfTokenGenerator = $csrfTokenGenerator;
    $this->moduleHandler = $moduleHandler;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Get the service.
   *
   * @return mixed
   *   The service.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks(): array {
    return ['renderEcoMode'];
  }

  /**
   * Check if the cookie exists.
   *
   * @return bool
   *   The cookie exists.
   */
  public function cookieExists(): bool {
    return !is_null($this->cookie);
  }

  /**
   * Get the new cookie object.
   *
   * @param int $value
   *   The cookie value.
   *
   * @return \Symfony\Component\HttpFoundation\Cookie
   *   The cookie object.
   */
  public function getNewCookieObject(int $value = 0): Cookie {
    return new Cookie($this->getCookieName(), $value, strtotime('+1 year'), '/', NULL, NULL, FALSE);
  }

  /**
   * Get the cookie name.
   *
   * @return string
   *   The cookie name.
   */
  public function getCookieName(): string {
    return self::ECO_MODE_COOKIE;
  }

  /**
   * Get the cookie value.
   *
   * @return float|bool|int|string|null
   *   The cookie value.
   */
  public function getCookieValue(): float | bool | int | string | null {
    return $this->cookie;
  }

  /**
   * Set the cookie value.
   *
   * @param float|bool|int|string|null $value
   *   The cookie value.
   *
   * @return float|bool|int|string|null
   *   The cookie value.
   */
  public function setCookieValue(float | bool | int | string | null $value): float | bool | int | string | null {
    return $this->cookie = $value;
  }

  /**
   * Set the cookie value.
   *
   * @return int
   *   The cookie value.
   */
  public function getSwitchEcoModeNewCookieValue(): int {
    return empty($this->cookie) ? 1 : 0;
  }

  /**
   * Get the cookie value.
   *
   * @return int
   *   The cookie value.
   */
  public function getEcoModeCookieValue(): int {
    return empty($this->cookie) ? 0 : 1;
  }

  /**
   * Get the request origin.
   *
   * @return string|null
   *   The request origin.
   */
  public function getRequestReferUrl(): ?string {
    return $this->request->getCurrentRequest()->headers->get('referer');
  }

  /**
   * Get the CSRF token.
   *
   * @return string
   *   The CSRF token.
   */
  public function getCsrfToken(): string {
    return $this->csrfTokenGenerator->get();
  }

  /**
   * Validate the CSRF token.
   *
   * @param string $token
   *   The CSRF token.
   *
   * @return bool
   *   The CSRF token is valid.
   */
  public function validateCsrfToken(string $token): bool {
    return $this->csrfTokenGenerator->validate($token);
  }

}
