<?php

namespace Drupal\eco_mode\Trait;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Trait RenderManagerTrait.
 *
 * Trait for render manager.
 *
 * @package Drupal\eco_mode\Trait
 */
trait RenderManagerTrait {

  /**
   * Render the element if eco-mode is off.
   *
   * @param array $elements
   *   The elements.
   * @param array $settings
   *   The settings.
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The items.
   * @param string $viewMode
   *   The view mode.
   *
   * @return array
   *   The element.
   *
   * @throws \Random\RandomException
   */
  public function renderEcoMode(array $elements, array $settings, FieldItemListInterface $items, string $viewMode): array {
    $elements = $this->getEcoModeCookieValue() && !\Drupal::requestStack()
      ->getCurrentRequest()->query->has('eco_ajax') ? $this->getReplacementElements($elements, $settings, $items, $viewMode) : $elements;
    $this->addCacheContexts($elements);
    $this->moduleHandler->alter('eco_mode_field_render', $elements, $settings);
    return $elements;
  }

  /**
   * Get the replacement elements.
   *
   * @param array $elements
   *   The elements.
   * @param array $settings
   *   The settings.
   * @param \Drupal\Core\Field\FieldItemListInterface $items
   *   The items.
   * @param string $viewMode
   *   The view mode.
   *
   * @return array
   *   The replacement elements.
   *
   * @throws \Random\RandomException
   */
  protected function getReplacementElements(array $elements, array $settings, FieldItemListInterface $items, string $viewMode): array {
    $entity = $items[0]->getEntity();
    $text = empty($settings['text']) ? '' : $settings['text'];
    $build = [];

    foreach ($elements as $delta => $element) {
      if (empty($settings['ajax'])) {
        $build[$delta] = $this->getNoLinkRender($text);
      }
      else {
        $build[$delta] = $this->getAjaxLinkRender($text, $entity, $items[0]->getFieldDefinition()
          ->getName(), $viewMode);
      }
    }

    return $build;
  }

  /**
   * Get the ajax link render.
   *
   * @param string $text
   *   The text.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity interface.
   * @param string $fieldName
   *   The field name.
   * @param string $viewMode
   *   The view mode.
   *
   * @return array
   *   The ajax link render.
   *
   * @throws \Random\RandomException
   */
  public function getAjaxLinkRender(string $text, EntityInterface $entity, string $fieldName, string $viewMode): array {
    $token = $this->generateRandomLinkToken();
    $url = Url::fromRoute('eco_mode.ajax_load_field', [
      'entity_type' => $entity->getEntityTypeId(),
      'entity_id' => $entity->id(),
      'field_name' => $fieldName,
      'view_mode' => $viewMode,
      'token' => $token,
      'eco_ajax' => TRUE,
    ]);

    $link = Link::fromTextAndUrl(t('@text', ['@text' => $text]), $url)
      ->toRenderable();
    $link['#attributes'] = [
      'class' => ['use-ajax', 'eco-mode-ajax'],
      'id' => 'eco-mode-ajax-field-link-' . $token,
    ];

    $link['#attached']['library'][] = 'core/drupal.ajax';

    return [
      '#theme' => 'eco_mode_ajax_link',
      '#link' => $link,
      '#entity_type' => $entity->getEntityTypeId(),
      '#entity_bundle' => $entity->bundle(),
      '#entity_id' => $entity->id(),
      '#field_name' => $fieldName,
      '#view_mode' => $viewMode,
    ];
  }

  /**
   * Get the no link render.
   *
   * @param string $text
   *   The text.
   *
   * @return array
   *   The no link render.
   */
  public function getNoLinkRender(string $text): array {
    return [
      '#theme' => 'eco_mode_no_link',
      '#text' => $text,
    ];
  }

  /**
   * Generate random link token.
   *
   * @return string
   *   The random link token.
   *
   * @throws \Random\RandomException
   *   The random exception.
   */
  protected function generateRandomLinkToken(): string {
    return bin2hex(random_bytes(4)) . dechex(random_int(1000, 9999));
  }

  /**
   * Add cache contexts.
   *
   * @param array $render
   *   The render array.
   */
  protected function addCacheContexts(array &$render): void {
    foreach ($render as $key => $value) {
      $render[$key]['#cache']['contexts'] = Cache::mergeContexts($render['#cache']['contexts'] ?? [], [
        'cookies:eco_mode',
        'url.query_args:eco_ajax',
      ]);
    }
  }

  /**
   * Render the field.
   *
   * @param string $entityType
   *   The entity type.
   * @param string|int $entityId
   *   The entity id.
   * @param string $fieldName
   *   The field name.
   * @param string $viewMode
   *   The view mode.
   *
   * @return array
   *   The rendered field.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function renderField(string $entityType, string | int $entityId, string $fieldName, string $viewMode): array {
    $entity = $this->entityTypeManager
      ->getStorage($entityType)
      ->load($entityId);

    if (!$entity || !$entity->hasField($fieldName)) {
      throw new NotFoundHttpException();
    }
    return $entity->$fieldName->view($viewMode);
  }

}
